@@ -17,7 +17,7 @@ sub write_string  {    #{{{
         = ($self->string_chunk_prefix(), $self->string_final_chunk_prefix());
     my @string_chunks = @{ $params->{chunks} };
     my $message = $self->hessianify_chunks( $prefixes, @string_chunks );
-    return $message;
+    return join "" => @{$message};
 }    #}}}
 
 sub write_xml {    #{{{
@@ -29,9 +29,19 @@ sub write_xml {    #{{{
         { prefix => 'x', last_prefix => 'X' }, 
         @xml_chunks 
       );
-    return $message;
+    return join "" => @{$message};
 }    #}}}
 
+sub  write_packet { #{{{
+    my ($self, $params) = @_;
+    my @packets = @{ $params->{packets}};
+    my $message = 
+    $self->hessianify_chunks({
+       prefix => "\x4f", last_prefix => "P" 
+        }, @packets);
+    return $message;
+} #}}}
+
 sub hessianify_chunks {    #{{{
     my ($self, $prefixes, @chunks ) = @_;
     my $last_chunk = pop @chunks;
@@ -43,8 +53,7 @@ sub hessianify_chunks {    #{{{
     my $last_prefix = $prefixes->{last_prefix};
     my $processed_last_chunk = ($last_prefix . write_chunk($last_chunk));
     push @message, $processed_last_chunk;
-    my $result = join "" => @message;
-    return $result;
+    return \@message;
 }    #}}}
 
 "one, but we're not the same";
@@ -64,14 +73,17 @@ Hessian::Serializer::String - Role for serialization of strings into hessian.
 
 =head1 INTERFACE
 
-=head2   hessianify_chunks
+=head2 hessianify_chunks
 
 
-=head2   write_chunk
+=head2 write_chunk
 
 
-=head2   write_string
+=head2 write_string
 
 
-=head2   write_xml
+=head2 write_xml
 
+=head2 write_packet
+
+Enclose a datastructure in packets for chunked transmission.
